package cv07;

import java.util.concurrent.atomic.AtomicInteger;

/**
 * @author Karel Cemus
 */
public class Guest implements Runnable {

    /**
     * jmeno hosta
     */
    private final String name;

    /**
     * pocet piv, ktere chce host vypit
     */
    private final int thirst;
    
    /** doba, jak dlouho host pije jedno pivo v milisekundach */
    private final long DRINK_TIME = 1000 + Math.round(Math.random() * 1000);

    /**
     * pocet piv, ktere host uz vypil
     */
    private int drunk = 0;

    /**
     * upozornil host cisnika? Chce si objednat?
     */
    private AtomicInteger wantToOrder = new AtomicInteger(0);

    /**
     * stul, u ktereho host sedi
     */
    private final Table table;

    public Guest(String name, int thirst, Table table) {
        this.name = name;
        this.thirst = thirst;
        this.table = table;
    }

    private void drinkBeer() throws InterruptedException {

        // vezmi si pivo ze stolu, pockej nez ho prinese
        table.take();

        System.out.println(String.format("[%s] Piju pivo.", this));

        // piti piva chvili trva
        Thread.currentThread().sleep(DRINK_TIME);

        drunk++;
        System.out.println(String.format("[%s] Pivo dopito.", this));
    }

    private void orderBeer() {

        // chci pivo!
        wantToOrder.incrementAndGet();
        System.out.println(String.format("[%s] Chci pivo.", this));

        // mavni na cisnika a pockej, dokud neprevezme objednavku
        Waiter.order(table);

        System.out.println(String.format("[%s] Cisnik upozornen.", this));
    }

    /**
     * chcete pivo?
     */
    public int takeOrder() {

        // objednavka
        int order = wantToOrder.get();

        if (order > 0) {
            System.out.println(String.format("[%s] Objednano %d piv.", this, order));
        }

        // vrat objednavku, pivo?
        return order;
    }

    private void sit() {
        table.takeSeat(this);
        System.out.println(String.format("[%s] Sedl jsem si ke stolu a chci %d piv.", this, thirst));
    }

    private void leave() {
        table.leave(this);
        System.out.println(String.format("[%s] Vstal jsem od stolu po te, co jsem vypil %d piv(a).", this, drunk));
    }

    @Override
    public String toString() {
        return name;
    }

    @Override
    public void run() {

        // host zije!
        System.out.println(String.format("[%s] Jsem tu a mam chut na %d piv(a).", this, thirst));

        // host prijde a sedne si ke stolu
        sit();

        try {
            // dokud ma host zizen
            while (thirst > drunk) {

                // objedna si pivo
                orderBeer();

                // vypije pivo
                drinkBeer();
            }
        } catch (InterruptedException ex) {
            System.err.println(String.format("[%s] Byl jsem vyrusen od piti piva! Odchazim bez placeni ...", this));
        }

        // host dopil a odchazi. plati cestou na baru
        leave();

        System.out.println(String.format("[%s] Uff, bylo to narocne, mam dost. Mejte se!", this));
    }
}
