package cv07;

import java.util.HashSet;
import java.util.Set;

/**
 * @author Karel Cemus
 */
public class Table {

    /** jmeno stolu */
    private final String name;

    /** hoste sedici u stolu */
    private final Set<Guest> guests = new HashSet<Guest>();

    /** pocet piv lezicich na stole */
    private int beers = 0;

    public Table( String name ) {
        this.name = name;
    }

    /** cisnik polozil pivo na stul */
    public void put( int beers ) {
        synchronized(this) {
            // polozime piva na stul
            this.beers += beers;
            
            // informujeme hosty o zmene
            notifyAll();
            
            // log
            System.out.println( String.format( "[%s] Obdrzeno %d piv.", this, beers ) );
        }
    }

    /** host si bere pivo ze stolu */
    public void take() throws InterruptedException {

        synchronized(this) {

            // dokud neni pivo, cekej
            while (beers == 0) {
                wait();
            }

            // vezmi si pivo, 100% je na stole
            beers--;
        }
    }

    /** host si bere misto u stolu */
    public void takeSeat( Guest guest ) {
        synchronized(this) {
            guests.add( guest );
            System.out.println( String.format( "[%s] Prisel host %s.", this, guest ) );
        }
    }

    /** host odchazi od stolu */
    public void leave( Guest guest ) {
        synchronized(this) {
            guests.remove( guest );
            System.out.println( String.format( "[%s] Odesel host %s.", this, guest ) );
        }
    }

    /** posbirat objednavky u stolu */
    public int takeOrder() {
        int beers = 0;
        synchronized(this){
            for(Guest guest:guests) beers += guest.takeOrder();
        }
        
        return beers;
    }

    @Override
    public String toString() {
        return name;
    }
}
