package cv07;

import java.util.*;

/**
 * @author Karel Cemus
 */
public class Waiter implements Runnable {

    /** doba cepovani jednoho piva v milisekundach */
    private static final int TAPPING_TIME = 500;

    /** fronta objednavek, dulezite je poradi, kazdy stul by zde mel byt maximalne jednou */
    private static final Queue<Table> queue = new LinkedList<Table>();

    /* mnozina stolu, ktere si chteji objednat */
    private static final Set<Table> tables = new HashSet<Table>();
    
    private static final Object lock = new Object();

    private final String name;

    public Waiter( String name ) {
        this.name = name;
    }

    public static void order( Table table ) {

        synchronized(lock) {
            // pokud stul jeste neceka na objednavku, pridame ho do fronty
            // pokud ceka, tak nic, vsak se docka
            if ( !tables.contains( table ) ) {
                // tento stul bude cekat na objednavku
                tables.add( table );
                // zaradime ho na konec fronty
                queue.add( table );
                // nejaky stul si chce objednat, upozorni cisnika, pokud se nejaky nudi
                lock.notify();
            }
        }
    }

    /** zjisti, kdo si chce objednat */
    private Table receiveOrder() throws InterruptedException {

        // stul, co si bude objednavat
        Table table;

        // pracujeme s objekty spolecnymi cele tride, musime ji zamknout
        synchronized(lock) {

             while(queue.isEmpty()) lock.wait();

            // 100% vime, ze fronta neni prazdna, vezmeme si cekajici stul
            table = queue.poll();
            
            // odstranime ho i z mnoziny cekajicich stolu
            tables.remove(table);
        }

        System.out.println( String.format( "[%s] Budu obsluhovat stul %s.", this, table ) );
        return table;
    }

    /** vzit objednavku z daneho stolu */
    private int takeOrder( Table table ) {
        int beers = table.takeOrder();
        System.out.println( String.format( "[%s] Prevzal jsem objednavku na %d piv od stolu %s", this, beers, table ) );
        return beers;
    }

    /** nacepovat objednana piva */
    private void tapBeer( int beers ) throws InterruptedException {
        for ( int i = 0; i < beers; ++i ) {
            // cepujeme pivo, to chvili trva
            Thread.currentThread().sleep(TAPPING_TIME);
        }
        System.out.println( String.format( "[%s] Nacepoval jsem %d piv.", this, beers ) );
    }

    /** dat piva na spravny stul */
    private void serve( Table table, int beers ) {
        table.put( beers );
        System.out.println( String.format( "[%s] polozil jsem %d piv na stul %s", this, beers, table ) );
    }

    @Override
    public void run() {

        System.out.println( String.format( "[%s] Smena zacala, snad to zvladnu.", this ) );

        try {
            // chudak cisnik, pracuje v nekonecne smycce
            while(true) {

                // zadost o ukonceni prace
                if(Thread.currentThread().isInterrupted()) break;

                // zjisti, ktery stul si chce objednat
                Table table = receiveOrder();

                // prevezmi objednavku
                int order = table.takeOrder();

                // natoc piva
                tapBeer(order);

                // odnes piva
                table.put(order);
            }

        } catch ( InterruptedException e ) {
            System.out.println( String.format( "[%s] Nekdo me vyrusil!", this ) );
        } finally {
            System.out.println( String.format( "[%s] Smena skoncila, jdu domu.", this ) );
        }
    }

    @Override
    public String toString() {
        return name;
    }
}
